
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "enums.h"
#include "cloudproviders.h"
#include "cloudproviderscollector.h"
#include "cloudprovidersaccount.h"
#include "cloudprovidersaccountexporter.h"
#include "cloudprovidersprovider.h"
#include "cloudprovidersproviderexporter.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "cloudprovidersaccount.h" */

GType
cloud_providers_account_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CLOUD_PROVIDERS_ACCOUNT_STATUS_INVALID), "CLOUD_PROVIDERS_ACCOUNT_STATUS_INVALID", "invalid" },
        { C_ENUM (CLOUD_PROVIDERS_ACCOUNT_STATUS_IDLE), "CLOUD_PROVIDERS_ACCOUNT_STATUS_IDLE", "idle" },
        { C_ENUM (CLOUD_PROVIDERS_ACCOUNT_STATUS_SYNCING), "CLOUD_PROVIDERS_ACCOUNT_STATUS_SYNCING", "syncing" },
        { C_ENUM (CLOUD_PROVIDERS_ACCOUNT_STATUS_ERROR), "CLOUD_PROVIDERS_ACCOUNT_STATUS_ERROR", "error" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("CloudProvidersAccountStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

