/* cloudproviderscollector.h
 *
 * Copyright (C) 2015 Carlos Soriano <csoriano@gnome.org>
 * Copyright (C) 2017 Julius Haertl <jus@bitgrid.net>
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CLOUD_PROVIDERS_COLLECTOR_H
#define CLOUD_PROVIDERS_COLLECTOR_H

#include <gio/gio.h>
#include "cloudprovidersaccount.h"

G_BEGIN_DECLS

#define CLOUD_PROVIDERS_TYPE_COLLECTOR (cloud_providers_collector_get_type())
G_DECLARE_FINAL_TYPE (CloudProvidersCollector, cloud_providers_collector, CLOUD_PROVIDERS, COLLECTOR, GObject)

CloudProvidersCollector *cloud_providers_collector_dup_singleton (void);
GList *cloud_providers_collector_get_providers (CloudProvidersCollector *self);
G_END_DECLS

#endif /* CLOUD_PROVIDERS_COLLECTOR_H */
